﻿using Microsoft.VisualStudio.TestTools.UITesting;
using Microsoft.VisualStudio.TestTools.UITesting.HtmlControls;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.CodedUi2012.CodedUiTests
{
    public static class BendayCodedUiExtensionMethods
    {
        public static void ChangeValue(this HtmlCheckBox control)
        {
            control.Checked = !control.Checked;
        }

        public static bool IsAnythingSelected(this UIListboxSelectedValueList control)
        {
            return IsAnythingSelected(GetItemsAsHtmlListItems(control.Items));
        }

        public static bool IsAnythingSelected(this HtmlComboBox control)
        {
            return IsAnythingSelected(GetItemsAsHtmlListItems(control.Items));
        }

        public static string GetCurrentValue(this HtmlComboBox control)
        {
            var selectedItem = GetSelectedItem(GetItemsAsHtmlListItems(control.Items));

            if (selectedItem == null)
            {
                return null;
            }
            else
            {
                return selectedItem.ValueAttribute;
            }
        }

        /// <summary>
        /// Gets the first selected item in the control.
        /// </summary>
        /// <param name="control"></param>
        /// <returns></returns>
        public static string GetCurrentValue(this UIListboxSelectedValueList control)
        {
            var selectedItem = GetSelectedItem(GetItemsAsHtmlListItems(control.Items));

            if (selectedItem == null)
            {
                return null;
            }
            else
            {
                return selectedItem.ValueAttribute;
            }
        }

        /// <summary>
        /// Gets a list of all selected values.
        /// </summary>
        /// <param name="control"></param>
        /// <returns></returns>
        public static List<string> GetCurrentValues(this UIListboxSelectedValueList control)
        {
            List<HtmlListItem> items = GetItemsAsHtmlListItems(control.Items);

            List<string> selectedValues = new List<string>();

            foreach (var item in items)
            {
                if (item.Selected == true)
                {
                    selectedValues.Add(item.ValueAttribute);
                }
            }

            return selectedValues;
        }

        public static void SelectEveryOtherItem(this UIListboxSelectedValueList control)
        {
            SelectEveryOtherItem(control, GetItemsAsHtmlListItems(control.Items));
        }

        public static void ToggleListBoxSelection(this UIListboxSelectedValueList control)
        {
            ToggleListBoxSelection(control, GetItemsAsHtmlListItems(control.Items));
        }

        public static void ChangeValue(this UIListboxSelectedValueList control)
        {
            List<HtmlListItem> items = GetItemsAsHtmlListItems(control.Items);

            bool isAnythingSelected = IsAnythingSelected(items);

            if (isAnythingSelected == false)
            {
                SelectEveryOtherItem(control, items);
            }
            else
            {
                ToggleListBoxSelection(control, items);
            }
        }

        public static void ChangeValue(this HtmlComboBox control)
        {
            var index = control.SelectedIndex;

            var numberOfItems = control.Items.Count;

            int newIndex;

            if (index + 1 < numberOfItems)
            {
                newIndex = index + 1;
            }
            else
            {
                newIndex = 0;
            }

            control.SelectedIndex = newIndex;
        }

        public static HtmlListItem GetSelectedItem(List<HtmlListItem> items)
        {
            foreach (var item in items)
            {
                if (item.Selected == true)
                {
                    return item;
                }
            }

            return null;
        }

        public static bool IsAnythingSelected(List<HtmlListItem> items)
        {
            if (GetSelectedItem(items) == null)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        public static void SelectEveryOtherItem(HtmlList control, List<HtmlListItem> items)
        {
            bool needsComma = false;
            bool selectThisOne = true;

            StringBuilder builder = new StringBuilder();

            foreach (var item in items)
            {
                if (selectThisOne == true)
                {
                    if (needsComma == true)
                    {
                        builder.Append(",");
                    }

                    builder.Append(item.DisplayText);

                    needsComma = true;
                }

                selectThisOne = !selectThisOne;
            }

            control.SelectedItemsAsString = builder.ToString();
        }

        public static void ToggleListBoxSelection(UIListboxSelectedValueList control, List<HtmlListItem> items)
        {
            bool needsComma = false;

            StringBuilder builder = new StringBuilder();

            foreach (var item in items)
            {
                if (item.Selected == false)
                {
                    if (needsComma == true)
                    {
                        builder.Append(",");
                    }

                    builder.Append(item.DisplayText);

                    needsComma = true;
                }
            }

            control.SelectedItemsAsString = builder.ToString();
        }

        public static List<HtmlListItem> GetItemsAsHtmlListItems(UITestControlCollection fromValues)
        {
            List<HtmlListItem> toValues = new List<HtmlListItem>();

            HtmlListItem toValue;

            foreach (var fromValue in fromValues)
            {
                toValue = fromValue as HtmlListItem;

                if (toValue == null)
                {
                    throw new InvalidOperationException("Could not convert item to HtmlListItem.");
                }

                toValues.Add(toValue);
            }

            return toValues;
        }
    }
}
